﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2015 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.IO;
using System.IO.Compression;

namespace HeuristicLab.Persistence {
  public abstract class Serializer : ISerializer {
    public virtual void Serialize(object o, Stream stream) {
      SerializeBundle(Mapper.ToBundle(o), stream);
    }
    public virtual void Serialize(object o, string path) {
      string tempfile = Path.GetTempFileName();

      using (FileStream stream = File.Create(tempfile)) {
        using (var zipStream = new DeflateStream(stream, CompressionMode.Compress)) {
          Serialize(o, zipStream);
        }
      }

      File.Copy(tempfile, path, true);
      File.Delete(tempfile);
    }
    public virtual byte[] Serialize(object o) {
      using (var memoryStream = new MemoryStream()) {
        using (var zipStream = new DeflateStream(memoryStream, CompressionMode.Compress)) {
          Serialize(o, zipStream);
        }
        return memoryStream.ToArray();
      }
    }
    protected abstract void SerializeBundle(Bundle bundle, Stream stream);

    public virtual object Deserialize(Stream stream) {
      return Mapper.ToObject(DeserializeBundle(stream));
    }
    public virtual object Deserialize(string path) {
      using (var fileStream = new FileStream(path, FileMode.Open)) {
        using (var zipStream = new DeflateStream(fileStream, CompressionMode.Decompress)) {
          return Deserialize(zipStream);
        }
      }
    }
    public virtual object Deserialize(byte[] data) {
      using (var memoryStream = new MemoryStream(data)) {
        using (var zipStream = new DeflateStream(memoryStream, CompressionMode.Decompress)) {
          return Deserialize(zipStream);
        }
      }
    }
    protected abstract Bundle DeserializeBundle(Stream stream);
  }
}
